from pathlib import Path
from typing import Optional
import jwt
import requests
import typer
from rich import print
from rich.syntax import Syntax

import os 
import json
import typing

from urllib.parse import urljoin

from . import get_token, get_project_id, get_org_id, BASE_URL, PATH_ARTIFACT_DIR
from joblib import Parallel, delayed

app = typer.Typer()

@app.command()
def details(model_name:str, model_version:str, name:str=''):
    '''This function returns the details of the artifact for a given model
    
    Parameters
    ----------
    model_name : str
        The name of the model you want to get the artifact details for
    model_version: str
        The version of the model
    name : str
        The name of the artifact.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()



    url_path_1 = '{}/project/{}/model/{}/artifacts/{}/'.format(org_id, project_id, model_name, name)
    url = urljoin(BASE_URL, url_path_1)

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }
    

    response = requests.get(url, headers=headers)


    if response.status_code == 200:
        res_text = json.loads(response.text)

        if len(res_text) == 0:
            print('[bold yellow] No Artifacts have been found for the model')
            print(res_text)
            return 
        else:
            print('[bold green]Artifacts have been found for the model')
            print(res_text)
            return res_text

    else:
        print('[bold red]Unable to obtain the artifact details')
        print(response.text)
        return




@app.command()
def add(artifact: str, name: str, model_name: str, model_version:str) -> str:    
    '''`add` function takes in the path of the artifact, name of the artifact and the model name and
    registers the artifact
    
    Parameters
    ----------
    artifact : str
        The path to the artifact file.
    name : str
        The name of the artifact.
    model_name : str
        The name of the model you want to add artifacts to.
    model_version: str
        The version of the model
    
    Returns
    -------
        The response is a JSON object
    
    '''
    
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()
    
    url_path_1 = '{}/project/{}/model/{}/artifacts/add'.format(org_id, project_id, model_name)
    url = urljoin(BASE_URL, url_path_1)


    user_token = get_token()
    project_id = get_project_id()

    headers = {
        'Authorization': 'Bearer {}'.format(user_token)
    }

    
    
    if os.path.isfile(artifact):
        file = {'file': (name, open(artifact, 'rb'))}
    else:
        print('[bold red] Artifact doesnot exist at the given path')

    
    data = {'name': name, 'path' : artifact}

    response = requests.post(url, data=data, files=file, headers=headers)
    
    
    if response.status_code == 200:
        print(f"[bold green]Artifacts have been registered!")

    else:
        print(f"[bold red]Artifacts have not been registered!")
        print(response.text)

    return response.text


@app.command()
def fetch(model_name: str, model_version:str, name:str = ''):
    '''It fetches the artifact from the server and stores it in the local directory
    
    Parameters
    ----------
    model_name : str
        The name of the model you want to fetch the artifact from.
    model_version: str
        The version of the model
    name : str
        The name of the artifact to be fetched. If not specified, all artifacts will be fetched.
    
    Returns
    -------
        The response text is being returned.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()


    def fetch_artifact(artifact_details: dict):

        url = artifact_details['location']
        file_path_temp = artifact_details['path']
        file_name = file_path_temp.split(os.path.sep)[-1]
        save_path = os.path.join(PATH_ARTIFACT_DIR, file_name)
        print('save path', save_path)

        name_fetched = artifact_details['artifact']


        headers = {
            'Content-Type': 'application/x-www-form-urlencoded',
            'Authorization': 'Bearer {}'.format(user_token)
        }
        
        print('Artifact url', url)

        # response = requests.get(url, headers=headers)
        response = requests.get(url)

        print(response.status_code)

        if response.status_code == 200:
            print('[bold green] Artifact {} has been fetched'.format(name_fetched))

            save_dir = os.path.dirname(save_path)

            os.makedirs(save_dir, exist_ok=True)

            artifact_bytes = response.content

            open(save_path, 'wb').write(artifact_bytes)


            print('[bold green] Artifact {} has been stored at {}'.format(name_fetched, save_path))
            
            return response.text
        else:
            print('[bold red] Unable to fetch the artifact')

            return response.text


    artifact_details = details(model_name=model_name, name=name, model_version=model_version)

    if artifact_details is None:
        return

    if type(artifact_details) is dict:

        res_text = fetch_artifact(artifact_details)

    elif type(artifact_details) is list:
        res_text = Parallel(n_jobs=-1)(delayed(fetch_artifact)(art_det) for art_det in artifact_details)


    return res_text
    





@app.command()
def delete(name:str, model_name:str,  model_version:str) -> str:
    '''`delete()` deletes an artifact from a model
    
    Parameters
    ----------
    name : str
        The name of the artifact you want to delete.
    model_name : str
        The name of the model you want to delete the artifact from
    model_version: str
        The version of the model
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()

    url_path_1 = '{}/project/{}/model/{}/artifacts/{}/delete'.format(org_id, project_id, model_name, name)
    url = urljoin(BASE_URL, url_path_1)

    
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }
    

    # artifact_details = details(model_name=model_name, artifact=artifact)

    # if artifact_details is None:
    #     print('[bold red] Unable to find artifact details')
    #     return


    response = requests.delete(url, headers=headers)


    if response.status_code == 200:
        print(f"[bold green]Artifact has been deleted")
        
    else:
        print(f"[bold red]Unable to delete artifact")

    return response.text




if __name__ == "__main__":
    app()












import typer
from pureml.config.parser import Config
# from pureml.trainer.train import Trainer
import os

app = typer.Typer()

# @app.command('generate')
# def generate(project : str = typer.Option(..., '--project'), 
#                 data : str = typer.Option(..., '--data'), 
#                 output_parameters: str = typer.Option(..., '--output_parameters')):
#     '''
#     Generates the config file from input parameters
#     '''
#     # pass

#     os.makedirs(project, exist_ok=True)
    
#     config_path = os.path.join(project, 'config.yaml')
    
#     config = Config(config_path=config_path)
    
#     config.create_config(project=project, label=output_parameters, data=data)
#     config.load_config()
#     config.generate_config()


    


# @app.command('train')
# def train(project : str = typer.Option(..., '--project')):
#     '''
#     Trains the model from the config file in project folder
#     '''
    
#     config_path = os.path.join(project, 'config.yaml')
#     config = Config(config_path=config_path)
#     config.load_config()
    
#     trainer = Trainer(label_header=config.label_header, data_path=config.data_path, project_path=config.project_folder, 
#                         engine=config.engine, model=config.model, model_parameters=config.model_parameters,
#                         optimize=config.optimize)

#     trainer.load_data()

#     trainer.fit()

#     # trainer.app.launch(share=True)




# @app.command('predict')
# def predict():
#     '''
#     Predicts the output of the model in the given project directory for the given data
#     '''
#     pass


# @app.command('launch_gradio')
# def launch_gradio(project : str = typer.Option(..., '--project')):
#     config_path = os.path.join(project, 'config.yaml')
#     config = Config(config_path=config_path)
#     config.load_config()
    
#     trainer = Trainer(label_header=config.label_header, data_path=config.data_path, project_path=config.project_folder)

#     trainer.load_data()
#     trainer.load_model_for_prediction()
    
#     trainer.create_gradio()

#     trainer.app.launch()


# if __name__ == "__main__":
#     app()




import typer
from rich import print
# from puretrainer.train import Trainer
import os
from dotenv import load_dotenv

load_dotenv()


import pureml.cli.auth as auth
import pureml.cli.init as init
import pureml.cli.secrets as secrets
import pureml.cli.project as project
import pureml.cli.model as model
import pureml.cli.params as params
import pureml.cli.metrics as metrics
import pureml.cli.artifacts as artifacts

app = typer.Typer()
app.add_typer(auth.app, name="auth")
app.add_typer(init.app, name="init")
app.add_typer(secrets.app, name="secrets")

app.add_typer(project.app, name="project")
app.add_typer(model.app, name="model")
app.add_typer(params.app, name="params")
app.add_typer(metrics.app, name="metrics")
app.add_typer(artifacts.app, name="artifacts")


@app.callback(no_args_is_help=True)
def validate_user_authentication(ctx: typer.Context):
    # print(ctx.invoked_subcommand)
    if ctx.invoked_subcommand in ['auth']:
        return
    # user_token = auth.auth_validate()
    return

@app.command('predict')
def predict():
    '''
    Predicts the output of the model in the given project directory for the given data
    '''
    print("authenticated")
    pass

if __name__ == "__main__":
    app()


from pathlib import Path
from typing import Optional
import jwt
import requests
import typer
from rich import print
from rich.syntax import Syntax

import os 
import json
import typing
from urllib.parse import urljoin

from . import get_token, get_project_id, get_org_id, BASE_URL, PATH_PURE_DIR


app = typer.Typer()


@app.command()
def add(metrics, model_name: str, model_version:str) -> str:
    '''`add()` takes a dictionary of metrics and a model name as input and returns a string
    
    Parameters
    ----------
    metrics
        a dictionary of metrics
    model_name : str
        The name of the model you want to add metrics to.
    model_version: str
        The version of the model
    
    Returns
    -------
        The response.text is being returned.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()
    

    url_path_1 = '{}/project/{}/model/{}/metrics/add'.format(org_id,project_id, model_name)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }

    metrics = json.dumps(metrics)
    data = {'model_name': model_name, 'metrics': metrics}

    response = requests.post(url, data=data, headers=headers)


    if response.status_code == 200:
        print(f"[bold green]Metrics have been registered!")

    
    else:
        print(f"[bold red]Metrics have not been registered!")
        
    return response.text


@app.command()
def fetch(model_name: str, model_version:str, metric:str='') -> str:
    '''This function fetches the metrics of a model
    
    Parameters
    ----------
    model_name : str
        The name of the model you want to fetch metrics for.
    model_version: str
        The version of the model
    metric : str
        The metric you want to fetch. If you want to fetch all the metrics, leave this parameter empty.
    
    Returns
    -------
        The metrics that are fetched
    
    '''
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()
    

    url_path_1 = '{}/project/{}/model/{}/metrics/{}'.format(org_id, project_id, model_name, metric)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    response = requests.get(url, headers=headers)

    if response.status_code == 200:
        res_text = json.loads(response.text)

        if metric == '':

            metrics = res_text

            print(f"[bold green]Metrics have been fetched")
            print(metrics)

            return metrics


        else:
            if 'metric' in res_text.keys() and 'value' in res_text.keys():
                metrics = res_text['value']
                # metrics = json.loads(metrics)

                print(f"[bold green]Metric has been fetched")
                print(res_text['metric'], ':', res_text['value'])

                return metrics

            else:
                print('[bold red]Metric {} is not available for the model!'.format(metric))
                print(response.text)
                return
        
            

    else:
        print(f"[bold red]Unable to fetch Metrics!")
        print(response.text)
        return


@app.command()
def delete(metric:str, model_name:str, model_version:str) -> str:
    '''This function deletes a metric from a model
    
    Parameters
    ----------
    model_name : str
        The name of the model you want to delete the metric from
    metric : str
        The name of the metric to delete
    model_version: str
        The version of the model
    
    '''
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()
    

    url_path_1 = '{}/project/{}/model/{}/metrics/{}/delete'.format(org_id, project_id, model_name, metric)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    response = requests.delete(url, headers=headers)

    if response.status_code == 200:
        print(f"[bold green]Metric has been deleted")
        
    else:
        print(f"[bold red]Unable to delete Metric")

    return response.text




if __name__ == "__main__":
    app()



# import os
# from pathlib import Path
# import joblib

from .models import load_model, save_model

from .cli import project as project
from .cli import model as model
from .cli import metrics as metrics
from .cli import params as params
from .cli import artifacts as artifacts



from .model_framework import ModelFramework, ModelFrameworkType


from pathlib import Path
from .packaging import Model

# MODEL_SAVE_BY_TYPE = {
#     ModelFrameworkType.SKLEARN: SKLearn().save_model
# }


# from .model_packaging.sklearn import SKLearn
# from .model_packaging.catboost import CatBoost
# from .model_packaging.xgboost import XGBoost
# from .model_packaging.lightgbm import LightGBM
# from .model_packaging.keras import Keras
# from .model_packaging.tensorflow import Tensorflow
# from .model_packaging.pytorch import Pytorch



# MODEL_FRAMEWORKS_BY_TYPE = {
#     ModelFrameworkType.SKLEARN: SKLearn(),
#     ModelFrameworkType.XGBOOST: XGBoost(),
#     ModelFrameworkType.LIGHTGBM: LightGBM(),
#     ModelFrameworkType.CATBOOST: CatBoost(),
#     ModelFrameworkType.KERAS: Keras(),
#     # ModelFrameworkType.HUGGINGFACE_TRANSFORMER: HuggingfaceTransformer(),
#     ModelFrameworkType.PYTORCH: Pytorch(),
# }


# SUPPORTED_MODEL_FRAMEWORKS = [
#     ModelFrameworkType.SKLEARN,
#     ModelFrameworkType.XGBOOST,
#     ModelFrameworkType.LIGHTGBM,
#     ModelFrameworkType.KERAS,
#     ModelFrameworkType.TENSORFLOW,
#     # ModelFrameworkType.HUGGINGFACE_TRANSFORMER,
# ]

def save_model(model, model_name) -> None: 
    ''' The function takes in a model and a model name, and saves the model to the default `models` directory. 
    
    Parameters
    ----------
    model
        The model you want to save.
    model_name
        The name of the model.
    
    '''
    Model(model=model, model_name=model_name).save_model()



def load_model(model_path:Path):
    ''' Loads a model from a given path
    
    Parameters
    ----------
    model_path : Path
        The path to the model you want to load.
    
    Returns
    -------
        The model is being returned.
    
    '''
    model = Model(model_path=model_path).load_model()

    return model




import yaml
import typer
from rich import print
from enum import Enum

app = typer.Typer()
    
class ExperimentTrackingService(Enum):
    kedro = "kedro"
    modeldb = "modeldb"
    mlflow = "mlflow"
    determined = "determined"
    dvc = "dvc"
    weightsandbiases = "weights&biases"
    neptune = "neptune"
    polyaxon = "polyaxon"
    clearml = "clearml"
    tensorboard = "tensorboard"
    none = "none"

class DataVersioningService(Enum):
    dolt = "dolt"
    dvc = "dvc"
    weightsandbiases = "weights&biases"
    neptune = "neptune"
    gitlfs = "gitlfs"
    pachyderm = "pachyderm"
    lakefs = "lakefs"
    none = "none"

class OrchestrationService(Enum):
    apache_airflow = "apache_airflow"
    argo_workflows = "argo_workflows"
    luigi = "luigi"
    kubeflow = "kubeflow"
    kedro = "kedro"
    nextflow = "nextflow"
    dagster = "dagster"
    apache_beam = "apache_beam"
    zenml = "zenml"
    flyte = "flyte"
    prefect = "prefect"
    ray = "ray"
    dvc = "dvc"
    polyaxon = "polyaxon"
    clearml = "clearml"
    pachyderm = "pachyderm"
    orchest = "orchest"
    mlrun = "mlrun"
    none = "none"

class ArtifactTrackingService(Enum):
    kubeflow = "kubeflow"
    mlflow = "mlflow"
    weightsandbiases = "weights&biases"
    neptune = "neptune"
    polyaxon = "polyaxon"
    clearml = "clearml"
    pachyderm = "pachyderm"
    mlrun = "mlrun"
    none = "none"

class ModelRegistryService(Enum):
    modeldb = "modeldb"
    mlflow = "mlflow"
    determined = "determined"
    weightsandbiases = "weights&biases"
    neptune = "neptune"
    clearml = "clearml"
    mlrun = "mlrun"
    none = "none"

class ModelServingService(Enum):
    seldoncore = "seldoncore"
    bentoml = "bentoml"
    nvidia_triton = "nvidia_triton"
    tensorflow_serving = "tensorflow_serving"
    kserve = "kserve"
    fastapi = "fastapi"
    torchserve = "torchserve"
    ray = "ray"
    cog = "cog"
    modeldb = "modeldb"
    mlflow = "mlflow"
    clearml = "clearml"
    nuclio = "nuclio"
    mlrun = "mlrun"
    none = "none"

class DataValidatorService(Enum):
    somedatavalidator = "somedatavalidator"
    none = "none"

@app.callback(invoke_without_command=True)
def init(
    ctx: typer.Context,
    name: str = typer.Option("my_pureml_project", "--name", prompt="• Enter project name", show_default=True),
    version: str = typer.Option("0.1.0", "--version", prompt="\n• Enter project version", show_default=True),
    experminent_tracking: ExperimentTrackingService = typer.Option("none", "--experiment_tracking", prompt="\n• Experiment tracking service\nOptions -", show_choices=True, show_default=True),
    data_versioning: DataVersioningService = typer.Option("none", "--data_versioning", prompt="\n• Data versioning service\nOptions -", show_choices=True, show_default=True),
    orchestration: OrchestrationService = typer.Option("none", "--orchestration", prompt="\n• Orchestration service\nOptions -", show_choices=True, show_default=True),
    artifact_tracking: ArtifactTrackingService = typer.Option("none", "--artifact_tracking", prompt="\n• Artifact_tracking service\nOptions -", show_choices=True, show_default=True),
    model_registry: ModelRegistryService = typer.Option("none", "--model_registry", prompt="\n• Model registry service\nOptions -", show_choices=True, show_default=True),
    model_serving: ModelServingService = typer.Option("none", "--model_serving", prompt="\n• Model serving service\nOptions -", show_choices=True, show_default=True),
    data_validator: DataValidatorService = typer.Option("none", "--data_validator", prompt="\n• Data validator service\nOptions -", show_choices=True, show_default=True),
) -> None:
    '''
    Initialize the project
    '''
    if ctx.invoked_subcommand is not None:
        return
    print("\nYour choices are - ")
    print(f":heavy_check_mark: [bold]Name[/bold] - {name}")
    print(f":heavy_check_mark: [bold]Version[/bold] - {version}")
    print(f":heavy_check_mark: [bold]Experiment tracking[/bold] - {experminent_tracking.value}")
    print(f":heavy_check_mark: [bold]Data versioning[/bold] - {data_versioning.value}")
    print(f":heavy_check_mark: [bold]Orchestration[/bold] - {orchestration.value}")
    print(f":heavy_check_mark: [bold]Artifact tracking[/bold] - {artifact_tracking.value}")
    print(f":heavy_check_mark: [bold]Model registry[/bold] - {model_registry.value}")
    print(f":heavy_check_mark: [bold]Model serving[/bold] - {model_serving.value}")
    print(f":heavy_check_mark: [bold]Data validator[/bold] - {data_validator.value}")

    data = {
        "name": name,
        "version": version,
        "experminent_tracking": experminent_tracking.value,
        "data_versioning": data_versioning.value,
        "orchestration": orchestration.value,
        "artifact_tracking": artifact_tracking.value,
        "model_registry": model_registry.value,
        "model_serving": model_serving.value,
        "data_validator": data_validator.value,
    }

    with open("pureml.yaml", "w") as write_file:
        yaml.dump(data, write_file, sort_keys=False)
    
    print("\n[green bold]:heavy_check_mark: Generated pureml.yaml")

if __name__ == "__main__":
    app()



import requests
import typer
from rich import print
from rich.syntax import Syntax

import os 
import json


from . import get_token, get_project_id, get_org_id, BASE_URL, PATH_PURE_DIR
from pureml import save_model, load_model
from urllib.parse import urljoin
import joblib

app = typer.Typer()


@app.command()
def list():
    '''This function will return a list of all the models in the project
    
    Returns
    -------
        A list of all the models in the project
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()

    url_path_1 = '{}/project/{}/models'.format(org_id, project_id)
    url = urljoin(BASE_URL, url_path_1)
    
    data = {"project_id": project_id}

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    response = requests.get(url,data=data, headers=headers)
    # print(response.text)
    if response.status_code == 200:
        print(f"[bold green]Obtained list of models")
        print(response.text)
    else:
        print(f"[bold red]Unable to obtain the list of models!")
        
    return response.text







@app.command()
def register(model, name:str, version:str='v1') -> str:
    ''' The function takes in a model, a name and a version and saves the model locally, then uploads the
    model to the PureML server
    
    Parameters
    ----------
    model
        The model you want to register
    name : str
        The name of the model.
    version: str, optional
        The version of the model.
    
    '''
        
    save_path = '.pureml'
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()


    url_path_1 = '{}/project/{}/model/create'.format(org_id, project_id)
    url = urljoin(BASE_URL, url_path_1)

    
    save_model(model, name)

    name_with_ext = '.'.join([name, 'pkl'])

    model_path = os.path.join(save_path, name_with_ext)

    headers = {
        'Authorization': 'Bearer {}'.format(user_token)
    }


    files = {'file': (name_with_ext, open(model_path, 'rb'))}

    data = {'name': name, 'project_id':project_id, 'version': version}
    response = requests.post(url, files=files, data=data, headers=headers)

    if response.status_code == 200:
        print(f"[bold green]Model has been registered!")

        return response.text
    else:
        print(f"[bold red]Model has not been registered!")
        print(response.status_code)
        print(response.text)
        return response.text


@app.command()
def details(name:str, version:str):
    '''It fetches the details of a model.
    
    Parameters
    ----------
    name : str
        The name of the model
    version: str
        The version of the model
    Returns
    -------
        The details of the model.
    
    '''
    
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()

    url_path_1 = '{}/project/{}/model/{}/details'.format(org_id, project_id, name)
    url = urljoin(BASE_URL, url_path_1)

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }
    

    response = requests.get(url, headers=headers)


    if response.status_code == 200:
        print(f"[bold green]Model details have been fetched")
        print(response.text)
        return response.text
    else:
        print(f"[bold red]Model details have not been found")
        return 



@app.command()
def fetch(name:str, version:str):
    '''This function fetches a model from the server and returns it as a `Model` object
    
    Parameters
    ----------
    name : str, optional
        The name of the model you want to fetch.
    version: str
        The version of the model
    
    Returns
    -------
        The model is being returned.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()


    url_path_1 = '{}/project/{}/model/create?name={}'.format(org_id, project_id, name)
    url = urljoin(BASE_URL, url_path_1)

    model_details = details(name=name, version=version)

    if model_details is None:
        return

    model_details = json.loads(model_details)
    model_location = model_details['location']
    model_url = 'http://{}'.format(model_location)
    
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }
    
    print('url', model_url)

    # response = requests.get(model_url, headers=headers)

    response = requests.get(model_url)

    if response.status_code == 200:
        model_bytes = response.content
        open('temp_model.pure', 'wb').write(model_bytes)

        model = load_model(model_path='temp_model.pure')


        print(f"[bold green]Model has been fetched")
        return model
    else:
        print(f"[bold red]Unable to fetch Model")
        print(response.status_code)
        print(response.text)
        print(response.url)
        return 



@app.command()
def delete(name:str, version:str) -> str:
    ''' This function deletes a model from the project
    
    Parameters
    ----------
    name : str
        The name of the model you want to delete
    version : str
        The version of the model to delete.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()


    url_path_1 = '{}/project/{}/model/{}/delete'.format(org_id, project_id, name)
    url = urljoin(BASE_URL, url_path_1)

    
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }

    data = {'version':version}
    
    
    response = requests.delete(url, headers=headers, data=data)


    if response.status_code == 200:
        print(f"[bold green]Model has been deleted")
        
    else:
        print(f"[bold red]Unable to delete Model")

    return response.text


@app.command()
def serve_model():
    pass



if __name__ == "__main__":
    app()


import os
from xmlrpc.client import Boolean
import requests
import typer
from rich import print
from rich.syntax import Syntax

app = typer.Typer()

import os 

from . import get_token, get_project_id, get_org_id, BASE_URL
from urllib.parse import urljoin



@app.command()
def details(project_id: str):
    '''It takes a project_id as input and returns the details of the project if it exists in the remote
    database
    
    Parameters
    ----------
    project_id : str
        The ID of the project you want to get details for.
    
    '''


    user_token = get_token()
    org_id = get_org_id()


    url_path_1 = '{}/project/id/{}'.format(org_id, project_id)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }

    response = requests.get(url,headers=headers)

    
    if response.status_code == 200:
        print(f"[bold green] Project exists in remote database")
        print(f"[bold green]", response.text)
    else:
        print(f"[bold green] Project doesnot exist in remote database")

    return response


def save_project(response:requests.Response):
    project_file_name = '.pureml/pure.project'
    project_path = os.path.join(os.getcwd(), project_file_name)

    project_dir = os.path.sep.join(project_path.split(os.path.sep)[:-1])
    os.makedirs(project_dir, exist_ok=True)

    with open(project_path, "w") as f:
        project_details: str = response.text
        project_details = project_details.strip('"')
        f.write(project_details)
    
    print(response.text)
    print('[bold green] Project Successfully created')






@app.command()
def create(name:str=None, description:str=None):
    '''It creates a project if it doesn't exist
    
    Parameters
    ----------
    name : str
        The name of the project.
    description : str
        The description of the project.
    
    '''


    user_token = get_token()
    org_id = get_org_id()
    
    url_path_1 = '{}/project/create'.format(org_id)
    url = urljoin(BASE_URL, url_path_1)


    if name is None:
        print(f"\n[bold] Enter project details[/bold]\n")

        name: str = typer.prompt("Project Name: ")
        description: str = typer.prompt("Description:")
    else:
        print("Project Name: ", name)
        print("Description:", description)
    


    data = {"name": name, "description": description}

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    project_id = get_project_id()
        
    project_response = details(project_id=project_id)
    
    if project_response.status_code == 200:
        print("[bold yellow] Project already exists. If you want to update the project, use update command")
        return
    else:
        response = requests.post(url,data=data, headers=headers)

        if response.status_code == 200:
            save_project(response=response)
        else:
            print(response.text)
            print('[bold red] Unable to create Project')

        return response.text
    




@app.command()
def delete(project_id:str = None):
    '''It deletes a project
    
    Parameters
    ----------
    project_id : str
        The id of the project you want to delete.
    
    '''


    user_token = get_token()
    org_id = get_org_id()

    if project_id is None:
        print(f"\n[bold]Enter Project Details[/bold]\n")
        project_id: str = typer.prompt("Project Id: ")
    

    url_path_1 = '{}/project/{}/delete'.format(org_id, project_id)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }

    def delete_project(url, headers):

        response = requests.delete(url, headers=headers)
        if response.status_code == 200:
            print(f"[bold green] Deleted the project!")
            print(response.text)
        else:
            print(f"[bold red] Could not delete the project!")
            print(response.status_code)
            print(response.text)

        return response


    project_reponse = details(project_id=project_id)

    if project_reponse.status_code == 200:
    
        update_response = delete_project(url=url, headers=headers)
        
        return update_response
    
    else:
        return



@app.command()
def update(project_id:str, name:str=None, description:str=''):
    '''It takes in a project id, a name and a description and updates the project with the given id
    
    Parameters
    ----------
    project_id : str
        The id of the project you want to update.
    name : str
        The name of the project.
    description : str
        The description of the project
    
    '''
    print(f"\n[bold]Enter the updated project details[/bold]\n")


    user_token = get_token()
    org_id = get_org_id()

    url_path_1 = '{}/project/{}/update'.format(org_id, project_id)
    url = urljoin(BASE_URL, url_path_1)



    if name is None:
        print(f"\n[bold] Enter project details[/bold]\n")

        name: str = typer.prompt("Project Name: ")
        description: str = typer.prompt("Description:")
    else:
        print("Project Name: ", name)
        print("Description:", description)

    
    data = {"name": name, "description": description}

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    def update_project(url, data, headers):
        response = requests.post(url,data=data, headers=headers)
        print(response.text)

        if response.status_code == 200:
            print(f"[bold green] Updated the project!")
            print(response.text)
        else:
            print(f"[bold red] Could not update the project!")
            print(response.text)
        
        return response

    project_reponse = details(project_id=project_id)

    if project_reponse.status_code == 200:
    
        update_response = update_project(url=url, data=data, headers=headers)
        
        return update_response
    
    else:
        return


@app.command()
def clone(project_id: str, overwrite:Boolean=None):
    ''' The function `clone` takes a project id and returns the project details
    
    Parameters
    ----------
    project_id : str
        The ID of the project you want to clone.
    overwrite : Boolean
        If True, the project will be overwritten if it already exists. If False, the project will not be
    overwritten. If None, the user will be prompted to overwrite the project if it already exists.
    
    Returns
    -------
        The project_response.text is being returned.
    
    '''
    
    project_reponse = details(project_id=project_id)

    save_project(response=project_reponse)


    return project_reponse.text

    

@app.command()
def list():
    '''`list()` is a function that calls the API to obtain a list of all projects
    
    Returns
    -------
        The response object is being returned.
    
    '''


    user_token = get_token()
    org_id = get_org_id()

    url_path_1 = '{}/project/all'.format(org_id)
    url = urljoin(BASE_URL, url_path_1)

    
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }

    response = requests.get(url, headers=headers)
    # print(response.text)

    if response.status_code == 200:
        print(f"[bold green] Obtained the project list")
        print(response.text)

        return response
    else:
        print(f"[bold red] Unable to obtain the project list!")
        return
    




if __name__ == "__main__":
    app()


from pureml.cli.model import register




def model(name:str, version:str):
    def decorator(func):
        print('Inside decorator')

        print('decorating', func, 'with argument', name, version)
        
        def wrapper(*args, **kwargs):
            print("Inside wrapper")
            
            model = func(*args, **kwargs)

            res_text = register(model=model, name=name, version=version)

            return res_text

        print("Outside  wrapper")

        return wrapper
    print('Outside decorator')
        
    return decorator


from pureml.cli.dataset import register




def dataset(name:str, version:str):

    def decorator(func):
        print('Inside decorator')

        print('decorating', func, 'with argument', name, version)
        
        def wrapper(*args, **kwargs):
            print("Inside wrapper")
            
            df = func(*args, **kwargs)

            res_text = register(dataset=df, name=name, version=version)

            return res_text

        print("Outside  wrapper")

        return wrapper
    print('Outside decorator')
        
    return decorator


from pydantic import BaseModel, root_validator

from abc import ABC, abstractmethod
from pathlib import Path
import typing

import yaml

from pydantic import BaseModel
from enum import Enum
import typing

from pureml.models.errors import FrameworkNotSupportedError
from .model_framework import ModelConfig, ModelFramework, ModelFrameworkType
# from . import MODEL_FRAMEWORKS_BY_TYPE, SUPPORTED_MODEL_FRAMEWORKS

from sklearn.base import BaseEstimator
from sklearn.pipeline import Pipeline
import pathlib



from .model_packaging.sklearn import SKLearn
from .model_packaging.catboost import CatBoost
from .model_packaging.xgboost import XGBoost
from .model_packaging.lightgbm import LightGBM
from .model_packaging.keras import Keras
from .model_packaging.tensorflow import Tensorflow
from .model_packaging.pytorch import Pytorch



MODEL_FRAMEWORKS_BY_TYPE = {
    ModelFrameworkType.SKLEARN: SKLearn(),
    ModelFrameworkType.XGBOOST: XGBoost(),
    ModelFrameworkType.LIGHTGBM: LightGBM(),
    ModelFrameworkType.CATBOOST: CatBoost(),
    ModelFrameworkType.KERAS: Keras(),
    # ModelFrameworkType.HUGGINGFACE_TRANSFORMER: HuggingfaceTransformer(),
    ModelFrameworkType.PYTORCH: Pytorch(),
}


SUPPORTED_MODEL_FRAMEWORKS = [
    ModelFrameworkType.SKLEARN,
    ModelFrameworkType.XGBOOST,
    ModelFrameworkType.LIGHTGBM,
    ModelFrameworkType.KERAS,
    ModelFrameworkType.TENSORFLOW,
    # ModelFrameworkType.HUGGINGFACE_TRANSFORMER,
]












class Model(ABC, BaseModel):
    model: typing.Any = None
    model_config: ModelConfig = ModelConfig(model=model)
    model_name: str = 'model'
    model_path: Path = None
    model_dir: str = Path('.pureml')
    model_class: str = None
    # model_framework: str= None


    @root_validator
    def set_fields(cls, values):
        # model = values.get('model')
        # model_name = values.get('model_name')
        model_config = values.get('model_config')
        # model_path = values.get('model_path')

        # print('model config')
        # print(model_config)

        # print('values')
        # print(values)

        if values['model_path'] is None:
            values['model_path'] = '.'.join([values['model_name'], 'pkl'])

        if values['model_name'] is not None:
            values['model_config'].model_name = values['model_name']

        if values['model'] is not None:
            values['model_config'].model = values['model']
            

        # print('model config')
        # print(model_config)

        # print('values')
        # print(values)


        return values



    def model_framework_from_model(self) -> ModelFramework:
        self.model_class = self.model.__class__
        model_framework = self.model_framework_from_model_class(self.model_class)

        return model_framework




    def model_framework_from_model_class(self, model_class) -> ModelFramework:

        for framework in MODEL_FRAMEWORKS_BY_TYPE.values():

            if framework.supports_model_class(model_class):
                return framework

        raise FrameworkNotSupportedError(
            "Model must be one of "
            + "/".join([t.value for t in SUPPORTED_MODEL_FRAMEWORKS])
        )


    def save_model(self):
        # print(self.model_config)
        model_framework = self.model_framework_from_model()
        # print(model_framework)
        # print('model class', self.model_class)

        self.model_config.model_framework = model_framework
        self.model_config.model_requirements = model_framework.get_requirements()
        
        
        # print(self.model_config)

        self.model_path = Path.joinpath(Path.cwd(), self.model_dir, self.model_path)
        self.model_path.parent.mkdir(parents=True, exist_ok=True)

        self.model_config.save_to_disk(model_config_path=self.model_path)

        print(self.model_path)


        # framework = self.model_framework.typ()
        # print(self.model_class, self.model_framework, framework)

        # MODEL_SAVE_BY_TYPE[framework](model, model_name)

        return self.model_path


    def load_model(self):
        self.model_config = self.model_config.load_from_disk(model_config_path=self.model_path)
        self.model = self.model_config.model

        return self.model
















import requests
import typer
from rich import print
from rich.syntax import Syntax

import os 
import json


from . import get_token, get_project_id, get_org_id, BASE_URL, PATH_DATASET_DIR
from urllib.parse import urljoin
import joblib
import pandas as pd


app = typer.Typer()



def save_dataset(dataset:pd.DataFrame, name:str):
    file_name = '.'.join([name, 'parquet'])
    save_path = os.path.join(PATH_DATASET_DIR, file_name)


    os.makedirs(PATH_DATASET_DIR, exist_ok=True)

    dataset.to_parquet(save_path)

    
    return save_path








@app.command()
def list():
    '''This function will return a list of all the datasets in the project
    
    Returns
    -------
        A list of all the datasets in the project
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()

    url_path_1 = '{}/project/{}/datasets'.format(org_id, project_id)
    url = urljoin(BASE_URL, url_path_1)
    
    data = {"project_id": project_id}

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    response = requests.get(url,data=data, headers=headers)
    # print(response.text)
    if response.status_code == 200:
        print(f"[bold green]Obtained list of datasets")
        print(response.text)
    else:
        print(f"[bold red]Unable to obtain the list of datasets!")
        
    return response.text






@app.command()
def register(dataset, name:str, version:str='v1') -> str:
    ''' The function takes in a dataset, a name and a version and saves the dataset locally, then uploads the
    dataset to the PureML server
    
    Parameters
    ----------
    dataset
        The dataset you want to register
    name : str
        The name of the dataset.
    version: str, optional
        The version of the dataset.
    
    '''
        
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()


    url_path_1 = '{}/project/{}/dataset/create'.format(org_id, project_id)
    url = urljoin(BASE_URL, url_path_1)

    
    save_path = save_dataset(dataset, name)

    name_with_ext = save_path.split('/')[-1]

    print('save path', save_path)
    print('name with ext', name_with_ext)


    headers = {
        'Authorization': 'Bearer {}'.format(user_token)
    }


    files = {'file': (name_with_ext, open(save_path, 'rb'))}

    data = {'name': name, 'project_id':project_id, 'version': version}
    response = requests.post(url, files=files, data=data, headers=headers)

    if response.status_code == 200:
        print(f"[bold green]Dataset has been registered!")

        return response.text
    else:
        print(f"[bold red]Dataset has not been registered!")
        print(response.status_code)
        print(response.text)
        return response.text



@app.command()
def details(name:str, version:str):
    '''It fetches the details of a dataset.
    
    Parameters
    ----------
    name : str
        The name of the dataset
    version: str
        The version of the dataset
    Returns
    -------
        The details of the dataset.
    
    '''
    
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()

    url_path_1 = '{}/project/{}/dataset/{}/details'.format(org_id, project_id, name)
    url = urljoin(BASE_URL, url_path_1)

    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }
    

    response = requests.get(url, headers=headers)


    if response.status_code == 200:
        print(f"[bold green]Dataset details have been fetched")
        print(response.text)
        return response.text
    else:
        print(f"[bold red]Dataset details have not been found")
        return 





@app.command()
def fetch(name:str, version:str):
    '''This function fetches a dataset from the server and returns it as a dataframe object
    
    Parameters
    ----------
    name : str, optional
        The name of the dataset you want to fetch.
    version: str
        The version of the dataset
    
    Returns
    -------
        The dataset dataframe is being returned.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()


    url_path_1 = '{}/project/{}/dataset/create?name={}'.format(org_id, project_id, name)
    url = urljoin(BASE_URL, url_path_1)

    dataset_details = details(name=name, version=version)

    if dataset_details is None:
        return

    dataset_details = json.loads(dataset_details)
    dataset_location = dataset_details['location']
    dataset_url = 'http://{}'.format(dataset_location)
    
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }
    
    print('url', dataset_url)

    # response = requests.get(dataset_url, headers=headers)

    response = requests.get(dataset_url)

    if response.status_code == 200:
        dataset_bytes = response.content
        open('temp_dataset.parquet', 'wb').write(dataset_bytes)

        dataset = pd.read_parquet('temp_dataset.parquet')


        print(f"[bold green]Dataset has been fetched")
        return dataset
    else:
        print(f"[bold red]Unable to fetch Dataset")
        print(response.status_code)
        print(response.text)
        print(response.url)
        return 






@app.command()
def delete(name:str, version:str) -> str:
    ''' This function deletes a dataset from the project
    
    Parameters
    ----------
    name : str
        The name of the dataset you want to delete
    version : str
        The version of the dataset to delete.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()


    url_path_1 = '{}/project/{}/dataset/{}/delete'.format(org_id, project_id, name)
    url = urljoin(BASE_URL, url_path_1)

    
    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }

    data = {'version':version}
    
    
    response = requests.delete(url, headers=headers, data=data)


    if response.status_code == 200:
        print(f"[bold green]Dataset has been deleted")
        
    else:
        print(f"[bold red]Unable to delete Dataset")

    return response.text







# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['pureml',
 'pureml.cli',
 'pureml.config',
 'pureml.decorators',
 'pureml.models',
 'pureml.models.model_packaging']

package_data = \
{'': ['*']}

install_requires = \
['PyJWT>=2.4.0,<3.0.0',
 'PyYAML>=6.0,<7.0',
 'numpy>=1.23.1,<2.0.0',
 'pydantic>=1.9.1,<2.0.0',
 'python-dotenv>=0.20.0,<0.21.0',
 'requests>=2.28.1,<3.0.0',
 'typer[all]>=0.6.1,<0.7.0']

entry_points = \
{'console_scripts': ['pure = pureml.cli.main:app']}

setup_kwargs = {
    'name': 'pureml',
    'version': '0.1.1',
    'description': '',
    'long_description': "# Pure\n\n\nPureML is the tool that lets you register, version, compare and stage models without the hassle of complex software. The core of PureML is the seamless collaboration across the team. \n\n\n# Documentation\n\nSee [PureML documentation](https://docs.pureml.com)\n\n\n\n# Installation\n\n## Get prerequisites\n* python versions `^3.8.0` are supported\n\n## Install package\n\n```bash\npip install pureml\n```\n\n\n# Getting Help\nIf you get stuck, don't worry we are here to help.\n\n- [PureML App](https://app.pureml.com)\n- [PureML Discord](https://discord.gg/xNUHt9yguJ)\n\n\n\n\n",
    'author': 'vamsidhar muthireddy',
    'author_email': 'vamsi.muthireddy@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://pureml.com/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path
from typing import Optional
import jwt
import requests
import typer
from rich import print
from rich.syntax import Syntax

import os 
import json
import typing
from urllib.parse import urljoin

from . import get_token, get_project_id, get_org_id, BASE_URL, PATH_PURE_DIR


app = typer.Typer()


@app.command()
def add(params, model_name: str, model_version:str) -> str:
    '''`add()` takes a dictionary of parameters and a model name as input and returns a string
    
    Parameters
    ----------
    params : dict
        a dictionary of parameters
    model_name : str
        The name of the model you want to add parameters to.
    model_version: str
        The version of the model
    
    Returns
    -------
        The response.text is being returned.
    
    '''

    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()
    
    url_path_1 = '{}/project/{}/model/{}/params/add'.format(org_id, project_id, model_name)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }

    params = json.dumps(params)
    data = {'model_name': model_name, 'params': params}

    response = requests.post(url, data=data, headers=headers)


    if response.status_code == 200:
        print(f"[bold green]Params have been registered!")

    else:
        print(f"[bold red]Params have not been registered!")

    
    return response.text
        


@app.command()
def fetch(model_name: str, model_version:str, param:str='') -> str:
    '''
    
    This function fetches the parameters of a model
    
    Parameters
    ----------
    model_name : str
        The name of the model you want to fetch the parameters for.
    model_version: str
        The version of the model
    param : str
        The name of the parameter to fetch. If not specified, all parameters are returned.
    
    Returns
    -------
        The params that are fetched
    
    '''
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()
    

    url_path_1 = '{}/project/{}/model/{}/params/{}'.format(org_id, project_id, model_name, param)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    response = requests.get(url, headers=headers)

#     params = None
    if response.status_code == 200:
        res_text = json.loads(response.text)

        if param == '':

            params = res_text

            print(f"[bold green]Params have been fetched")
            print(params)

            return params


        else:
            if 'param' in res_text.keys() and 'value' in res_text.keys():
                params = res_text['value']
                # params = json.loads(params)

                print(f"[bold green]Params have been fetched")
                print(res_text['param'], ':', res_text['value'])

                return params

            else:
                print('[bold red]Param {} are not available for the model!'.format(param))
                print(response.text)
                return
        
            

    else:
        print(f"[bold red]Unable to fetch Params!")
        print(response.text)
        return


@app.command()
def delete(param:str, model_name:str, model_version:str) -> str:
    '''This function deletes a parameter from a model
    
    Parameters
    ----------
    model_name : str
        The name of the model you want to delete the parameter from.
    param : str
        The name of the parameter to delete.
    model_version: str
        The version of the model
    
    '''
    user_token = get_token()
    org_id = get_org_id()
    project_id = get_project_id()
    

    url_path_1 = '{}/project/{}/model/{}/params/{}/delete'.format(org_id,project_id, model_name, param)
    url = urljoin(BASE_URL, url_path_1)


    headers = {
        'Content-Type': 'application/x-www-form-urlencoded',
        'Authorization': 'Bearer {}'.format(user_token)
    }


    response = requests.delete(url, headers=headers)

    if response.status_code == 200:
        print(f"[bold green]Param has been deleted")
        
    else:
        print(f"[bold red]Unable to delete Param")

    return response.text




if __name__ == "__main__":
    app()